## -*-Tcl-*-
 # ###################################################################
 #  Alpha - new Tcl folder configuration
 # 
 #  FILE: "alphaDefinitions.tcl"
 #                                    created: 98-04-05 21.31.20 
 #                                last update: 3/2/1999 {10:12:19 pm} 
 #  Author: Vince Darley
 #  E-mail: <darley@fas.harvard.edu>
 #    mail: Division of Engineering and Applied Sciences, Harvard University
 #          Oxford Street, Cambridge MA 02138, USA
 #     www: <http://www.fas.harvard.edu/~darley/>
 #  
 # Reorganisation carried out by Vince Darley with much help from Tom 
 # Fetherston, Johan Linde and suggestions from the Alpha-D mailing list.  
 # Alpha is shareware; please register with the author using the register 
 # button in the about box.
 #  
 # ###################################################################
 ##

proc alpha::getDefinitions {} {}
namespace eval keys {}
namespace eval global {}
namespace eval win {}

# Default settings of some global parameters (not preferences settings)
# These will all be over-written by user-defs if set.
set resumeRevert 0
set prefixString ">\ "
set suffixString "\ <--"
# mapping of windows to current modes.
set win::Modes("") ""
# keep count of number of dirty windows.
set win::NumDirty 0
set win::Current ""
# For mark stack.
set markName 0
set markStack ""
set winMenu "263"
set keyboard "U.S."
set oldkeyboard ""

win::SetProportions

lunion ftpSigs Arch FTCh Woof
lunion texSigs OTEX *TEX *XeT MPS*
lunion browserSigs MOSS MSIE dogz OlG1 HTVW
lunion httpDownloadSigs MOSS MSIE dogz OlG1 Geni Arch
lunion tclshSigs WIsH TclL Geni

array set keys::specialProcs { 
    "Next Stop" "ring::+"
    "Next Stop Or Indent" "bind::IndentOrNextstop"
    "Complete" "bind::Completion"
    "Complete Or Tab" "bind::TabOrComplete"
    "Prev Stop" "ring::-"
    "Real Tab" "insertActualTab"
    "nth Stop" "ring::nth"
    "Clear All Stops" "ring::clear"
}

array set keys::specialBindings {
    "Complete" "<O/c"
    "Prev Stop" "<U/c"
    "Real Tab" "<I/c"
    "nth Stop" "<B/c"
    "Clear All Stops" "<U<B/c"
    "Next Stop" ""
    "Complete Or Tab" ""
    "Next Stop Or Indent" "/c"
}

# Note: the Mercutio MDEF can only handle icon-suite resources
# with id's from 208 to 208+255 = 463.  Hence many of the little
# icons which Alpha contains cannot appear in menus.  You could
# of course do a little hacking....
set alpha::_icons {
    {DanR "Think Reference" 265}
    {OTEX "OzTeX" 266}
    {*TEX "TeXtures" 267}
    {XXXX "LaTeX" 270}
    {*XeT "CMacTeX" 272}
    {TeX+ "DirectTeX Pro" 299}
    {CWIE "Codewarrior" 268}
    {dogz "Cyberdog" 281}
    {Vbib "BibTeX" 282}
    {SLab "Scilab" 283}
    {IGR0 "Igor Pro" 284}
    {JAVC "Apple Applet Viewer" 285}
    {MOSS "Netscape Navigator" 293}
    {MOSS "Netscape Communicator" 294}
    {MSIE "Microsoft Internet Explorer" 295}
    {OlG1 "MacLynx" 296}
    {DanR "Think Ref Viewer" 310}
    {gsVR "Ghostview" 311}
    {PnLF "Finger" 313}
    {RZMI "MakeIndex" 314}
    {FTCh "Fetch" 315}
    {TGE+ "Tarmac" 316}
    {Gzip "Gzip" 317}
    {DStf "DropStuff" 318}
    {SITx "StuffIt Expander" 319}
    {ALTV "Programmer's Assistant" 400}
    {MPAD "Mupad" 411}
    {GPSE "Gnuplot" 415}
    {Vodo "VOODOO" 500}
}

namespace eval remote {}

ensureset "remote::site(Alpha's Home)" \
  "ftp://alpha.olm.net/pub/packages/"
ensureset "remote::site(Alpha's Upload site)" \
  "ftp://alpha.olm.net/incoming/"
ensureset "remote::site(Vince's Alpha-D site)" \
  "ftp://ftp.ucsd.edu/alpha/"

ensureset remote::listing {}

namespace eval keyboard {}

# array set keyboard::localized {
#     Svensk Swedish
#     U.S. U.S.
# }
# 
# # Works with MacOS 8.5, but possibly not earlier releases.
# if {![catch {aebuild::result 'MACS' kybd ckyl} __keyboard] && \
#   [info exist keyboard::localized($__keyboard)]} {
#     set keyboard [set keyboard::localized($__keyboard)]
# }
# 
# catch {unset keyboard::localized __keyboard}

set "keyboards(Australian)" {
 {1234567890-=[];'\`,./}
 {!@#$%^&*()_+{}:"|~<>?}
 <U/[
 <U/]
 {'' 0x0a '' 0x0a}
}
set "keyboards(Brasil)" {
 {1234567890'+~\<,.-}
 {!"#$%&/()=?*`^|>;:_}
 <I<U/8
 <I<U/9
 {'' 0x0a '' 0x0a '' 0x21 '' 0x21 '' 0x29 '' 0x29 '' 0x1e '`' 0x1e '~' 0x27 '^' 0x27}
}
set "keyboards(British)" {
 {1234567890-=[];'\`,./}
 {!@$%^&*()_+{}:"|~<>?}
 <U/[
 <U/]
 {'' 0x0a '' 0x0a '' 0x14}
}
set "keyboards(Canadian - CSA)" {
 {/1234567890-=^;,.}
 {\!@#$%?&*()_+:'"}
 <I/7
 <I/8
 {'' 0x1e '' 0x1e '' 0x27 '' 0x27 '' 0x2a '' 0x2a '' 0x2c '' 0x2c '' 0x32 '' 0x32 '^' 0x21 '' 0x21 '7' 0x1a '8' 0x1c}
}
set "keyboards(Canadian - ISO)" {
 {1234567890-=^;,.}
 {!"#$%?&*()_+:'.}
 <I/7
 <I/0
 {'' 0x0a '' 0x0a '' 0x1e '' 0x1e '' 0x27 '' 0x27 '' 0x2a '' 0x2a '' 0x2c '' 0x2c '' 0x32 '' 0x32 '^' 0x21 '' 0x21 '7' 0x1a '0' 0x1d}
}
set "keyboards(Canadian - French)" {
 {<1234567890-='[;`/,.}
 {>!@#$%?&*()_+"]:^|<>}
 <I/[
 <I<U/[
 {'' 0x2c '' 0x2c '' 0x06 '' 0x06 '`' 0x27 '^' 0x27}
}
set "keyboards(Danish)" {
 {$1234567890+'<,.-}
 {!"#%&/()=?`^*>;:_}
 <I<U/8
 <I<U/9
 {'' 0x0a '' 0x15 '' 0x21 '' 0x21 '' 0x29 '' 0x29 '' 0x27 '' 0x27 '' 0x1e '^' 0x1e '' 0x18 '`' 0x18}
}
set "keyboards(Dutch)" {
 {1234567890-=[];'\`,./}
 {!@#$%^&*()_+{}:"|~<>?}
 <U/[
 <U/]
 {'' 0x0a '' 0x0a}
}
set "keyboards(Espaol - ISO)" {
 {1234567890'`+<,.-}
 {!"$%&/()=?^*>;:_}
 <I/
 <I/
 {'' 0x2a '' 0x29 '' 0x2a '' 0x29 '' 0x18 '' 0x18 '' 0x0a '' 0x0a '' 0x14 '`' 0x21 '^' 0x21 '' 0x27 '' 0x27}
}
set "keyboards(Finnish)" {
 {1234567890+'<,.-}
 {!"#%&/()=?`^*>;:_}
 <I<U/8
 <I<U/9
 {'' 0x0a '' 0x0a '' 0x15 '' 0x21 '' 0x27 '' 0x29 '' 0x21 '' 0x27 '' 0x29 '' 0x1e '^' 0x1e '' 0x18 '`' 0x18}
}
set "keyboards(Flemish)" {
 {@&"'(!)-^$`<,;:=}
 {#1234567890_*%>?./+}
 <I/(
 <I/)
 {'' 0x13 '(' 0x17 '' 0x16 '' 0x1a '' 0x19 '' 0x1d ')' 0x1b '' 0x1b '' 0x27 '^' 0x21 '' 0x21 '`' 0x2a '' 0x2a}
}
set "keyboards(French)" {
 {@&"'(!)-^$`<,;:=}
 {#1234567890_*%>?./+}
 <I/(
 <I/)
 {'' 0x13 '(' 0x17 '' 0x16 '' 0x1a '' 0x19 '' 0x1d ')' 0x1b '' 0x1b '' 0x27 '^' 0x21 '' 0x21 '`' 0x2a '' 0x2a}
}
set "keyboards(French - numerical)" {
 {@&"'(!)-^$`<,;:=}
 {#1234567890_*%>?./+}
 <I/(
 <I/)
 {'' 0x13 '(' 0x17 '' 0x16 '' 0x1a '' 0x19 '' 0x1d ')' 0x1b '' 0x1b '' 0x27 '^' 0x21 '' 0x21 '`' 0x2a '' 0x2a}
}
set "keyboards(German)" {
 {^1234567890+#<,.-}
 {!"$%&/()=?`*^>;:_}
 <I/8
 <I/9
 {'^' 0x0a '' 0x0a '' 0x14 '' 0x21 '' 0x29 '' 0x27 '' 0x21 '' 0x29 '' 0x27 '' 0x1b '' 0x18 '`' 0x18 '8' 0x1c '9' 0x19}
}
set "keyboards(Italian)" {
 {@&"'()-=$<,;:}
 {#1234567890_+^*%>?./!}
 <I/(
 <I/)
 {'(' 0x15 ')' 0x1a '' 0x17 '' 0x16 '' 0x1c '' 0x19 '' 0x1d '' 0x21 '' 0x27 '' 0x2c '' 0x2a '' 0x2a}
}
set "keyboards(Norwegian)" {
 {'1234567890+@<,.-}
 {!"#$%&/()=?`^*>;:_}
 <I<U/8
 <I<U/9
 {'' 0x0a '' 0x21 '' 0x27 '' 0x29 '' 0x21 '' 0x27 '' 0x29 '' 0x1e '^' 0x1e '' 0x18 '`' 0x18}
}
set "keyboards(Spanish)" {
 {[1234567890-=`;'<,.}
 {]!#$%/&*()_+:">?}
 <I<U/<
 <U/[
 {'' 0x12 '' 0x21 '' 0x21 '' 0x29 '' 0x29 '' 0x2b '' 0x2c '' 0x2c '`' 0x1e '' 0x1e}
}
set "keyboards(Swedish)" {
 {1234567890+'<,.-}
 {!"#%&/()=?`^*>;:_}
 <I<U/8
 <I<U/9
 {'' 0x0a '' 0x0a '' 0x15 '' 0x21 '' 0x21 '' 0x27 '' 0x27 '' 0x29 '' 0x29 '' 0x1e '^' 0x1e '' 0x18 '`' 0x18}
}
set "keyboards(Swiss French)" {
 {1234567890'^$<,.-}
 {+"*%&/()=?`!>;:_}
 <I/8
 <I/9
 {'' 0x0a '' 0x0a '' 0x21 '' 0x21 '' 0x29 '' 0x29 '' 0x27 '' 0x27 '' 0x15 '' 0x2a '' 0x1e '!' 0x1e '^' 0x18 '`' 0x18 '8' 0x1c '9' 0x19}
}
set "keyboards(Swiss German)" {
 {1234567890'^$<,.-}
 {+"*%&/()=?`!>;:_}
 <I/8
 <I/9
 {'' 0x0a '' 0x0a '' 0x21 '' 0x21 '' 0x29 '' 0x29 '' 0x27 '' 0x27 '' 0x15 '' 0x2a '' 0x1e '!' 0x1e '^' 0x18 '`' 0x18 '8' 0x1c '9' 0x19}
}
set "keyboards(U.S.)" {
 {1234567890-=[];'\`,./}
 {!@#$%^&*()_+{}:"|~<>?}
 <U/[
 <U/]
 {'' 0x0a '' 0x0a}
}

set "keyboards(Slovenian)" {
  {1234567890/+<,.-}
  {!"#$%&'()=?*Ʈ>;:_}
  <U<I/
  <U<I/
  {'' 0x0a '' 0x0a '' 0x21 '' 0x21 '' 0x1e '' 0x1e '' 0x29 '' 0x29 '' 0x27 '' 0x27 '' 0x2a '' 0x2a}
}

set "keyboards(Croatian)" {
  {1234567890/+䶋<,.-}
  {!"#$%&'()=?*ᭉ>;:_}
  <U<I/
  <U<I/
  {'' 0x0a '' 0x0a '' 0x21 '' 0x21 '' 0x1e '' 0x1e '' 0x29 '' 0x29 '' 0x27 '' 0x27 '' 0x2a '' 0x2a}
}

set "keyboards(Roman - JIS)" {
  {1234567890-^@[;:]`,./}
  {!"#$%&'()0=~`{+*}~<>?}
 <U/[
 <U/]
 {'' 0x0a '' 0x0a}
}

if {![llength ${global::features}]} {
    set global::features [list internationalMenus filesetMenu \
      internetConfigMenu eudoraMenu]
    if {!$skipPrefs} {
	if {[dialog::yesno "Alpha contains a lot of useful additional functionality\
	  in the form of menus, packages and features.  Many of these provide\
	  basic things like completions, a recent files menu, keyboard macros,\
	  electric code insertion...\r\r  Would you like me to activate\
	  the standard feature set?  (Either\
	  way you can turn them on and off using the\
	  'Config->Preferences->Menus And Features' menu item)"]} {
	    lappend global::features recentFilesMenu elecCompletions \
	      elecExpansions macros elecBindings emacs autoContinueComment
	}
    }
}
